#!/usr/bin/python
import pybase64
import cv2
import json
import logging
import os
import pika
import sys
import time
import traceback
import numpy as np
from absl import app, flags
from absl.flags import FLAGS
from io import BytesIO
from logger import get_logger

os.environ["OPENCV_FFMPEG_CAPTURE_OPTIONS"] = "rtsp_transport;udp"

flags.DEFINE_string("camera_ip", "192.168.1.108", "camera ip")
flags.DEFINE_string("wps_topic", "WPS_LPR", "WPS topic for each cam")
flags.DEFINE_string("user", "wps", "camera or DVR user")
flags.DEFINE_string("password", "wpsbrasil2021", "Password for rtsp cam")
flags.DEFINE_string("channel_dvr", "1", "Channel DVR Cam")
flags.DEFINE_string("broker", "localhost", "Broker Address Rabbit MQ")
flags.DEFINE_integer("frame_ocr_rate", 12, "select which frame rate to perform ocr")
flags.DEFINE_integer("roi_x", "0", "Region of Interess X")
flags.DEFINE_integer("roi_y", "0", "Region of Interess Y")
flags.DEFINE_integer("roi_size_x", "1280", "Size X")
flags.DEFINE_integer("roi_size_y", "720", "Size Y")
flags.DEFINE_boolean("hikvision", False, "Hikvision Camera")
flags.DEFINE_boolean("pumatronix", False, "Pumatronix Camera")
flags.DEFINE_boolean("tplink", False, "TP Link Camera")
flags.DEFINE_boolean('video', False, 'Test video')


def restart_program():
    python = sys.executable
    os.execl(python, python, * sys.argv)


def main(_argv):
    logger = get_logger(__name__)
    logging.getLogger("pika").setLevel(logging.WARNING)

    camera_ip = FLAGS.camera_ip
    channel_dvr = FLAGS.channel_dvr
    frame_ocr_rate = FLAGS.frame_ocr_rate
    user = FLAGS.user
    password = FLAGS.password
    broker = FLAGS.broker
    roi_size_x = FLAGS.roi_size_x
    roi_size_y = FLAGS.roi_size_y
    roi_x = FLAGS.roi_x
    roi_y = FLAGS.roi_y
    wps_topic = FLAGS.wps_topic

    connection = pika.BlockingConnection(pika.ConnectionParameters(host=broker))
    channel = connection.channel()
    channel.queue_declare(queue='wps_lpr', durable=True)

    if FLAGS.hikvision:
        video_path = "rtsp://"+user+":"+password+"@"+camera_ip+":554/Streaming/Channels/"+channel_dvr+"01"
    elif FLAGS.pumatronix:
        video_path = "rtsp://"+camera_ip+":7051/mjpeg"
    elif FLAGS.tplink:
        video_path = "rtsp://"+user+":"+password+"@"+camera_ip+":554/stream1"
    elif FLAGS.video:
        video_path = "/WPSBrasil/lpr-wps/video.mp4"
    else:
        video_path = "rtsp://"+user+":"+password+"@"+camera_ip+":554/cam/realmonitor?channel="+channel_dvr+"&subtype=0"
        os.system('wget -q --delete-after '
                  f'"http://{user}:{password}@{camera_ip}/cgi-bin/configManager.cgi?action=setConfig'
                  f'&Encode[0].MainFormat[0].Video.FPS={frame_ocr_rate}'
                  f'&Encode[0].MainFormat[0].Video.BitRate=2048'
                  f'&Encode[0].MainFormat[0].Video.BitRateControl=CBR'
                  f'&Encode[0].MainFormat[0].Video.GOP={frame_ocr_rate}"'
                  )

    logger.info(f"Conectando camera: {camera_ip}:{channel_dvr}")
    vid = cv2.VideoCapture(video_path, cv2.CAP_FFMPEG)

    run = True
    while run:
        frame_num = 0
        while True:
            return_value, frame = vid.read()
            if return_value:
                try:
                    frame_num += 1
                    if frame_num == 1:
                        logger.info(f"Camera conectada")
                    if frame_num % frame_ocr_rate == 0:
                        start = time.perf_counter()
                        end_frame_color = time.perf_counter()
                        # frame = frame[roi_y:roi_y+roi_size_y, roi_x:roi_x+roi_size_x]
                        end_frame_roi = time.perf_counter()
                        frame_bytes = BytesIO()
                        end_frame_to_bytes = time.perf_counter()
                        np.save(frame_bytes, frame, allow_pickle=True)
                        end_frame_np_array = time.perf_counter()
                        frame_send = frame_bytes.getvalue()
                        end_frame_send = time.perf_counter()
                        image_to_ocr = {
                                "wps_topic": wps_topic,
                                "imagem": pybase64.b64encode(frame_send).decode(),
                                "roi_size_x": roi_size_x,
                                "roi_size_y": roi_size_y,
                                "roi_x": roi_x,
                                "roi_y": roi_y
                        }
                        end_dict = time.perf_counter()
                        channel.basic_publish(exchange='',
                                              routing_key='wps_lpr',
                                              properties=pika.BasicProperties(expiration='2000',),
                                              body=json.dumps(image_to_ocr))
                        end_rabbitmq_publish = time.perf_counter()
                        total_time = round((end_rabbitmq_publish - start) * 1000, 3)
                        total_end_frame_color = round((end_frame_color - start) * 1000, 3)
                        total_end_frame_roi = round((end_frame_roi - end_frame_color) * 1000, 3)
                        total_end_frame_to_bytes = round((end_frame_to_bytes - end_frame_roi) * 1000, 3)
                        total_end_frame_np_array = round((end_frame_np_array - end_frame_to_bytes) * 1000, 3)
                        total_end_frame_send = round((end_frame_send - end_frame_to_bytes) * 1000, 3)
                        total_end_dict = round((end_dict - end_frame_send) * 1000, 3)
                        total_rabbimq_publish = round((end_rabbitmq_publish - end_dict) * 1000, 3)
                        logger.debug(
                            f"{f'CV2_Color: {total_end_frame_color}ms'} "
                            f"{f'|ROI: {total_end_frame_roi}ms'} "
                            f"{f'|Image to Bytes: {total_end_frame_to_bytes}ms'} "
                            f"{f'|NP Array: {total_end_frame_np_array}ms'} "
                            f"{f'|Frame Send: {total_end_frame_send}ms'}"
                            f"{f'|Python Dict: {total_end_dict}ms'}"
                            f"{f'|Json and Rabbitmq Publish: {total_rabbimq_publish}ms'} "
                            f"{f'{chr(10)} Total: {total_time}ms'}")

                except Exception:
                    logger.warn(traceback.format_exc())
                    logger.warn("Erro ao processar frame.")
                    break
            else:
                logger.error("Conexao com a camera perdida, nova conexao em 30seg.")
                time.sleep(30)
                logger.info("tentando nova conexao")
                restart_program()
                pass


if __name__ == "__main__":
    try:
        app.run(main)
    except SystemExit:
        pass
