#!/usr/bin/python
import pybase64
import logging
import json
import pika
import time
from io import BytesIO
from recognize import Recognize
import paho.mqtt.publish as publish
import numpy as np
from logger import get_logger


class Consumer:

    def __init__(self):

        self.recognize = Recognize()

    def run(self):

        logger = get_logger(__name__)
        logging.getLogger("pika").setLevel(logging.WARNING)
        connection = pika.BlockingConnection(pika.ConnectionParameters(host='localhost'))
        channel = connection.channel()
        channel.queue_declare(queue='wps_lpr', durable=True)

        while True:
            def callback(ch, method, properties, body):
                msg = json.loads(body)
                try:
                    starting_time = time.perf_counter()
                    consumer = msg
                    time_consumer = time.perf_counter()
                    frame = consumer["imagem"]
                    time_frame_consumer = time.perf_counter()
                    frame = pybase64.b64decode(frame)
                    time_frame_b64_decode = time.perf_counter()
                    wps_topic = consumer["wps_topic"]
                    time_wps_topic = time.perf_counter()
                    stream = BytesIO(frame)
                    time_stream = time.perf_counter()
                    image = np.load(stream, allow_pickle=True)
                    time_convert_image = time.perf_counter()
                    result = self.recognize.start(image, wps_topic)

                    tempo_result = time.perf_counter()
                    time_consumer_final = round((time_consumer - starting_time) * 1000, 5)
                    time_frame_consumer_final = round((time_frame_consumer - time_consumer) * 1000, 5)
                    time_frame_b64_decode_final = round((time_frame_b64_decode - time_frame_consumer) * 1000, 5)
                    time_wps_topic_final = round((time_wps_topic - time_frame_b64_decode) * 1000, 5)
                    time_stream_final = round((time_stream - time_wps_topic) * 1000, 5)
                    time_convert_image_final = round((time_convert_image - time_stream) * 1000, 5)
                    tempo_result_final = round((tempo_result - time_convert_image) * 1000, 5)

                    logger.debug(
                        f'{f"{wps_topic}| "} '
                        f'{f"Tempo Consumer: {time_consumer_final}| "} '
                        f'{f"Tempo Frame: {time_frame_consumer_final}| "}'
                        f'{f"Tempo Base 64 Decode: {time_frame_b64_decode_final}| "} '
                        f'{f"Tempo WPS_Topic: {time_wps_topic_final}| "} '
                        f'{f"Tempo Stream:{time_stream_final}| "} '
                        f'{f"Tempo Convert Image: {time_convert_image_final}| "}'
                        f'{f"Tempo Result: {tempo_result_final}"}')

                    if result:
                        publish.single(wps_topic, payload=json.dumps(result), hostname="localhost",
                                       auth={'username': "wps", 'password': "wps@2021"})
                        elapsed_time = time.perf_counter()
                        tempo_mqtt_publish = round((elapsed_time - tempo_result) * 1000, 3)
                        total_time = round((elapsed_time - starting_time) * 1000, 3)

                        if 'placa_lpr' in result:
                            logger.info("%s %s" % (
                                f"{wps_topic}| ", f"Placa: {result['placa_lpr']}"))
                            logger.debug("%s %s %s" % (
                                f"Tempo de Model WPS_LPR {result['time_ocr']}| ",
                                f"Tempo total Recognize: {result['time_total_model']}| ",
                                f"\nConfianca Placa: {result['confianca']}"))

                    logger.debug(
                        f'{f"Tempo MQTT Publish: {tempo_mqtt_publish}| "}'
                        f'{f"Tempo total: {total_time}"}'
                        f'{chr(10)}'
                    )
                except:
                    pass

            channel.basic_consume(queue='wps_lpr', on_message_callback=callback, auto_ack=True)
            channel.start_consuming()


def main():
    Consumer().run()


if __name__ == "__main__":
    main()
