#!/usr/bin/python
from silence_tensorflow import silence_tensorflow
silence_tensorflow()

import cv2
import io
import numpy as np
import pybase64
import time
import tensorflow as tf

from core import utils
from datetime import datetime
from logger import get_logger
from pytz import timezone
from tensorflow.python.saved_model import tag_constants
from turbojpeg import TurboJPEG

logger = get_logger(__name__)
jpeg = TurboJPEG()

physical_devices = tf.config.experimental.list_physical_devices("GPU")
if len(physical_devices) > 0:
    tf.config.experimental.set_memory_growth(physical_devices[0], True)


input_size = 416
saved_model_loaded = tf.saved_model.load(
    "detect-plate-v3", tags=[tag_constants.SERVING]
)
infer = saved_model_loaded.signatures["serving_default"]
logger.debug(f"Modelo de detecção carregado")
ocr_model_path = "ocr_model_v3"
ocr_model = tf.keras.models.load_model(ocr_model_path, compile=False)
logger.debug(f"Modelo de OCR carregado")
logger.info("WPS_LPR carregado \n \n")

overlay = cv2.imread('data/wps_logo.png', cv2.IMREAD_UNCHANGED)
alpha_channel = overlay[:, :, 3] / 255
overlay_colors = overlay[:, :, :3]
alpha_mask = np.dstack((alpha_channel, alpha_channel, alpha_channel))
h, w = overlay.shape[:2]


class Recognize:

    def start(self, image, camera_name):
        starting_time = time.perf_counter()
        starting_time_plate_model = time.perf_counter()

        #frame_orig = image
        #roi_size_x, roi_size_y, roi_x, roi_y = roi[0], roi[1], roi[2], roi[3]
        #image = image[roi_y:roi_y+roi_size_y, roi_x:roi_x+roi_size_x]
        batch_data, image_orig = utils.image_preprocess(np.copy(image), [input_size, input_size])
        pred_result = infer(batch_data)
        bboxes = utils.process_detection_prediction(pred_result['pred_bbox'], image_orig, input_size)
        pred_bbox = utils.get_pred_bbox_to_recognition(bboxes)
        end_time_plate_model = time.perf_counter()

        starting_time_ocr_model = time.perf_counter()
        image, plate_number, confidence = utils.draw_bbox(
            image, pred_bbox, ocr_model, plate=True
        )
        end_time_ocr_model = time.perf_counter()

        total_time_plate_model = round((end_time_plate_model - starting_time_plate_model) * 1000, 3)
        total_time_ocr_model = round((end_time_ocr_model - starting_time_ocr_model) * 1000, 3)
        end_time_wps_lpr = round((end_time_ocr_model - starting_time) * 1000, 3)


        starting_time_resize_img = time.perf_counter()
        dim = (720, 480)
        result = cv2.resize(image, dim, cv2.INTER_AREA)
        if plate_number and len(plate_number) == 7:
            x1 = (result.shape[1] / dim[0]) * 120
            y1 = (result.shape[0] / dim[1]) * 30
            cv2.rectangle(result, (10, 10), (int(x1), int(y1)), (0, 85, 255), -1)
            cv2.putText(result, "{}".format(plate_number), (10, int(y1) - 5), cv2.FONT_HERSHEY_COMPLEX_SMALL,
                        int(x1) / 110, (0, 0, 0), 2)
        end_time_resize_img = time.perf_counter()

        starting_time_logo_wps = time.perf_counter()
        y, x = result.shape[:2]
        x = x - 10
        background_subsection = result[10:h + 10, x - w:x]
        composite = background_subsection * (1 - alpha_mask) + overlay_colors * alpha_mask
        result[10:h + 10, x - w:x] = composite
        end_time_logo_wps = time.perf_counter()

        starting_time_encode_img = time.perf_counter()
        result = jpeg.encode(result)
        starting_time_io_Bytes = time.perf_counter()
        result = io.BytesIO(result)
        starting_time_io_Buffer = time.perf_counter()
        result = io.BufferedReader(result)
        starting_time_base64 = time.perf_counter()
        image = pybase64.b64encode(result.read()).decode()
        end_time_encode_img = time.perf_counter()

        time_now = datetime.now(timezone("Brazil/East"))
        timestamp = time_now.isoformat()
        end_time_recognize = time.perf_counter()
        total_time_recognize = round((end_time_recognize - starting_time) * 1000, 3)

        starting_time_payload = time.perf_counter()
        payload = {
            "time_ocr": end_time_wps_lpr,
            "time_total_model": total_time_recognize,
            "placa_lpr": plate_number,
            "confianca": round((sum(confidence)) / 7, 3),
            "timestamp": timestamp,
            "idCamera": camera_name,
            "possiveisPlacas": [
                {
                    "placa": plate_number,
                    "confiancaPlaca": round((sum(confidence)) / 7, 3),
                    "confiancaCaractere": confidence,
                }
            ],
            "imagem": {
                "type": "image/jpg",
                "data": image,
            },
        }
        end_time_payload = time.perf_counter()

        starting_time_payload_2 = time.perf_counter()
        payload_no_plate = {
            "time_ocr": total_time_ocr_model,
            "time_total_model": total_time_recognize,
            "timestamp": timestamp,
            "imagem": {
                "type": "image/jpg",
                "data": image,
            },
        }
        end_time_payload_2 = time.perf_counter()

        total_time_payload_2 = round((end_time_payload_2 - starting_time_payload_2) * 1000, 3)
        total_time_payload = round((end_time_payload - starting_time_payload) * 1000, 3)
        total_time_encode_img = round((starting_time_io_Bytes - starting_time_encode_img) * 1000, 3)
        total_time_encode_io_Bytes = round((starting_time_io_Buffer - starting_time_io_Bytes) * 1000, 3)
        total_time_encode_io_Buffer = round((starting_time_base64 - starting_time_io_Buffer) * 1000, 3)
        total_time_encode_Base64 = round((end_time_encode_img - starting_time_base64) * 1000, 3)
        total_time_logo_wps = round((end_time_logo_wps - starting_time_logo_wps) * 1000, 3)
        total_time_resize_img = round((end_time_resize_img - starting_time_resize_img) * 1000, 3)

        logger.debug(
            f'{f"Tempo plate model: {total_time_plate_model}| "}'
            f'{f"Tempo OCR model: {total_time_ocr_model}| "}'
            f'{f"Resize: {total_time_resize_img}| "}'
            f'{f"Logo: {total_time_logo_wps}| "}'
            f'{f"Encode JPG: {total_time_encode_img}| "}'
            f'{f"IO Bytes: {total_time_encode_io_Bytes}| "}'
            f'{f"IO Buffer: {total_time_encode_io_Buffer}| "}'
            f'{f"Base64 Encode: {total_time_encode_Base64}| "}'
            f'{f"Payload: {total_time_payload}| "}'
            f'{f"Payload_no_Plate {total_time_payload_2}"}')

        if plate_number:
            if len(plate_number) == 7 and sum(confidence) > 6.1:
                confianca_caractere_min = ([0.75, 0.75, 0.75, 0.9, 0.75, 0.9, 0.9])
                conf_min_char = np.less_equal(confidence, confianca_caractere_min)
                conf_char = sum(conf_min_char) != 0
                if conf_char == 0:
                    return payload
                else:
                    return payload_no_plate
            else:
                return payload_no_plate
        else:
            return payload_no_plate
